<?php

class Banner extends dbConnection
{
    public $id;
    public $title;
    public $image;
    public $type;
    public $order;
    public $link; // Added property for link
    public $date_added;

    // ... (existing methods)

    public function addBannerToDB()
    {
        try {
            $sql = "INSERT INTO tabl_banners (title, image, type, `order`, link, date_added) VALUES (:title, :image, :type, :order, :link, NOW())";
            $params = [
                ':title' => $this->title,
                ':image' => $this->image,
                ':type' => $this->type,
                ':order' => $this->order,
                ':link' => $this->link
            ];

            $this->dbQuery($sql, $params);

            $this->id = $this->dbInsertId();

            $this->date_added = date("Y-m-d H:i:s");

            return $this->id;
        } catch (PDOException $e) {
            return "Error adding banner: " . $e->getMessage();
        }
    }

    public function editBannerInDB()
    {
        try {
            $sql = "UPDATE tabl_banners SET title = :title, image = :image, type = :type, `order` = :order, link = :link WHERE id = :id";
            $params = [
                ':id' => $this->id,
                ':title' => $this->title,
                ':image' => $this->image,
                ':type' => $this->type,
                ':order' => $this->order,
                ':link' => $this->link
            ];

            $this->dbQuery($sql, $params);

            return true;
        } catch (PDOException $e) {
            return "Error editing banner: " . $e->getMessage();
        }
    }

    public function removeBannerFromDB()
    {
        try {
            // Get banner data for image removal
            $bannerData = $this->getBannerByIdFromDB($this->id);

            if ($bannerData) {
                $imagePath = "../../assets/images/banner/" . $bannerData['image'];

                // Remove banner image from server
                if (file_exists($imagePath)) {
                    unlink($imagePath);
                }
            }

            // Remove banner from the database
            $sql = "DELETE FROM tabl_banners WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true;
        } catch (PDOException $e) {
            return "Error removing banner: " . $e->getMessage();
        }
    }

    public function getBannerByIdFromDB($id)
    {
        try {
            // Retrieve banner information from the 'tabl_banners' table by ID
            $sql = "SELECT * FROM tabl_banners WHERE id = :id";
            $params = [':id' => $id];

            $result = $this->dbQuery($sql, $params);

            if ($result !== false) {
                $bannerData = $this->dbFetchAssoc($result);

                if ($bannerData) {
                    // Set banner data to class properties
                    $this->id = $bannerData['id'];
                    $this->title = $bannerData['title'];
                    $this->image = $bannerData['image'];
                    $this->type = $bannerData['type'];
                    $this->link = $bannerData['link'];
                    $this->order = $bannerData['order'];
                    $this->date_added = $bannerData['date_added'];

                    return true; // Successfully retrieved banner data
                } else {
                    return "Banner not found";
                }
            } else {
                return "Error retrieving banner data";
            }
        } catch (PDOException $e) {
            return "Error retrieving banner data: " . $e->getMessage();
        }
    }


    public function getBannersByTypeFromDB($type)
    {
        try {
            $sql = "SELECT * FROM tabl_banners WHERE type = :type ORDER BY `order` ASC";
            $params = [':type' => $type];

            $result = $this->dbQuery($sql, $params);

            $banners = [];

            while ($bannerData = $this->dbFetchAssoc($result)) {
                $banner = new Banner();
                $banner->id = $bannerData['id'];
                $banner->title = $bannerData['title'];
                $banner->image = $bannerData['image'];
                $banner->type = $bannerData['type'];
                $banner->link = $bannerData['link'];
                $banner->order = $bannerData['order'];
                $banner->date_added = $bannerData['date_added'];

                $banners[] = $banner;
            }

            return $banners;
        } catch (PDOException $e) {
            return "Error retrieving banners by type: " . $e->getMessage();
        }
    }

    public function getAllBannersFromDB()
    {
        try {
            $sql = "SELECT * FROM tabl_banners ORDER BY `order` ASC";
            $result = $this->dbQuery($sql);

            $banners = [];

            while ($bannerData = $this->dbFetchAssoc($result)) {
                $banner = new Banner();
                $banner->id = $bannerData['id'];
                $banner->title = $bannerData['title'];
                $banner->image = $bannerData['image'];
                $banner->type = $bannerData['type'];
                $banner->link = $bannerData['link'];
                $banner->order = $bannerData['order'];
                $banner->date_added = $bannerData['date_added'];

                $banners[] = $banner;
            }

            return $banners;
        } catch (PDOException $e) {
            return "Error retrieving all banners: " . $e->getMessage();
        }
    }
}
