<?php

class MainCategory extends dbConnection
{
    public $id;
    public $name;
    public $image;
    public $date_added;

    // Setter methods

    public function addMainCategoryToDB()
    {
        try {
            // Add main category information to the 'tabl_main_categories' table
            $sql = "INSERT INTO tabl_main_categories (name, image, date_added) VALUES (:name, :image, NOW())";
            $params = [':name' => $this->name, ':image' => $this->image];

            $this->dbQuery($sql, $params);

            $this->id = $this->dbInsertId(); // Get the ID of the newly inserted main category

            // Set date_added property
            $this->date_added = date("Y-m-d H:i:s");

            return $this->id; // Return the ID of the newly inserted main category
        } catch (PDOException $e) {
            return "Error adding main category: " . $e->getMessage();
        }
    }

    public function editMainCategoryInDB()
    {
        try {
            // Edit main category information in the 'tabl_main_categories' table
            $sql = "UPDATE tabl_main_categories SET name = :name, image = :image WHERE id = :id";
            $params = [':id' => $this->id, ':name' => $this->name, ':image' => $this->image];

            $this->dbQuery($sql, $params);

            return true; // Successfully edited the main category
        } catch (PDOException $e) {
            return "Error editing main category: " . $e->getMessage();
        }
    }

    public function deleteMainCategoryFromDB()
    {
        try {
            // Delete main category information from the 'tabl_main_categories' table
            $sql = "DELETE FROM tabl_main_categories WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true; // Successfully deleted the main category
        } catch (PDOException $e) {
            return "Error deleting main category: " . $e->getMessage();
        }
    }

    public function getAllMainCategoriesFromDB()
    {
        try {
            // Retrieve all main categories from the 'tabl_main_categories' table
            $sql = "SELECT * FROM tabl_main_categories";
            $result = $this->dbQuery($sql);

            $mainCategories = [];

            while ($mainCategoryData = $this->dbFetchAssoc($result)) {
                $mainCategory = new MainCategory();
                $mainCategory->id = $mainCategoryData['id'];
                $mainCategory->name = $mainCategoryData['name'];
                $mainCategory->image = $mainCategoryData['image'];
                $mainCategory->date_added = $mainCategoryData['date_added'];

                $mainCategories[] = $mainCategory;
            }

            return $mainCategories;
        } catch (PDOException $e) {
            return "Error retrieving main categories: " . $e->getMessage();
        }
    }



    public function getMainCategoryByIdFromDB($mainCategoryId)
    {
        try {
            // Retrieve main category information from the 'tabl_main_categories' table by ID
            $sql = "SELECT * FROM tabl_main_categories WHERE id = :id";
            $params = [':id' => $mainCategoryId];

            $result = $this->dbQuery($sql, $params);

            if ($result !== false) {
                $mainCategoryData = $this->dbFetchAssoc($result);

                if ($mainCategoryData) {
                    // Set main category data to class properties
                    $this->id = $mainCategoryData['id'];
                    $this->name = $mainCategoryData['name'];
                    $this->image = $mainCategoryData['image'];
                    $this->date_added = $mainCategoryData['date_added'];

                    return true; // Successfully retrieved main category data
                } else {
                    return "Main category not found";
                }
            } else {
                return "Error retrieving main category data";
            }
        } catch (PDOException $e) {
            return "Error retrieving main category data: " . $e->getMessage();
        }
    }

    public function getChildCategoriesOfMainCatFromDB($mainCategoryId)
    {
        try {
            // Retrieve child categories of a main category from the 'tabl_categories' table
            $sql = "SELECT * FROM tabl_categories WHERE main_cat_id = :main_cat_id";
            $params = [':main_cat_id' => $mainCategoryId];

            $result = $this->dbQuery($sql, $params);

            $childCategories = [];

            while ($childCategoryData = $this->dbFetchAssoc($result)) {
                $childCategory = new Category();
                $childCategory->id = $childCategoryData['id'];
                $childCategory->name = $childCategoryData['name'];
                $childCategory->image = $childCategoryData['image'];
                $childCategory->main_cat_id = $childCategoryData['main_cat_id'];
                $childCategory->date_added = $childCategoryData['date_added'];

                $childCategories[] = $childCategory;
            }

            return $childCategories;
        } catch (PDOException $e) {
            return "Error retrieving child categories: " . $e->getMessage();
        }
    }
}

class Category extends dbConnection
{
    public $id;
    public $name;
    public $image;
    public $main_cat_id;
    public $date_added;

    // Setter methods

    public function addCategoryToDB()
    {
        try {
            // Add category information to the 'tabl_categories' table
            $sql = "INSERT INTO tabl_categories (name, image, main_cat_id, date_added) VALUES (:name, :image, :main_cat_id, NOW())";
            $params = [':name' => $this->name, ':image' => $this->image, ':main_cat_id' => $this->main_cat_id];

            $this->dbQuery($sql, $params);

            $this->id = $this->dbInsertId(); // Get the ID of the newly inserted category

            // Set date_added property
            $this->date_added = date("Y-m-d H:i:s");

            return $this->id; // Return the ID of the newly inserted category
        } catch (PDOException $e) {
            return "Error adding category: " . $e->getMessage();
        }
    }

    public function editCategoryInDB()
    {
        try {
            // Edit category information in the 'tabl_categories' table
            $sql = "UPDATE tabl_categories SET name = :name, image = :image, main_cat_id = :main_cat_id WHERE id = :id";
            $params = [':id' => $this->id, ':name' => $this->name, ':image' => $this->image, ':main_cat_id' => $this->main_cat_id];

            $this->dbQuery($sql, $params);

            return true; // Successfully edited the category
        } catch (PDOException $e) {
            return "Error editing category: " . $e->getMessage();
        }
    }

    public function deleteCategoryFromDB()
    {
        try {
            // Delete category information from the 'tabl_categories' table
            $sql = "DELETE FROM tabl_categories WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true; // Successfully deleted the category
        } catch (PDOException $e) {
            return "Error deleting category: " . $e->getMessage();
        }
    }

    public function getAllCategoriesFromDB()
    {
        try {
            // Retrieve all categories from the 'tabl_categories' table
            $sql = "SELECT * FROM tabl_categories";
            $result = $this->dbQuery($sql);

            $categories = [];

            while ($categoryData = $this->dbFetchAssoc($result)) {
                $category = new Category();
                $category->id = $categoryData['id'];
                $category->name = $categoryData['name'];
                $category->image = $categoryData['image'];
                $category->main_cat_id = $categoryData['main_cat_id'];
                $category->date_added = $categoryData['date_added'];

                $categories[] = $category;
            }

            return $categories;
        } catch (PDOException $e) {
            return "Error retrieving categories: " . $e->getMessage();
        }
    }


    public function getCategoryByIdFromDB($categoryId)
    {
        try {
            // Retrieve category information from the 'tabl_categories' table by ID
            $sql = "SELECT * FROM tabl_categories WHERE id = :id";
            $params = [':id' => $categoryId];

            $result = $this->dbQuery($sql, $params);

            if ($result !== false) {
                $categoryData = $this->dbFetchAssoc($result);

                if ($categoryData) {
                    // Set category data to class properties
                    $this->id = $categoryData['id'];
                    $this->name = $categoryData['name'];
                    $this->image = $categoryData['image'];
                    $this->main_cat_id = $categoryData['main_cat_id'];
                    $this->date_added = $categoryData['date_added'];

                    return true; // Successfully retrieved category data
                } else {
                    return "Category not found";
                }
            } else {
                return "Error retrieving category data";
            }
        } catch (PDOException $e) {
            return "Error retrieving category data: " . $e->getMessage();
        }
    }

    public function getChildSubCategoriesOfCatFromDB($categoryId)
    {
        try {
            // Retrieve child subcategories of a category from the 'tabl_subcategories' table
            $sql = "SELECT * FROM tabl_subcategories WHERE cat_id = :cat_id";
            $params = [':cat_id' => $categoryId];

            $result = $this->dbQuery($sql, $params);

            $childSubCategories = [];

            while ($childSubCategoryData = $this->dbFetchAssoc($result)) {
                $childSubCategory = new SubCategory();
                $childSubCategory->id = $childSubCategoryData['id'];
                $childSubCategory->name = $childSubCategoryData['name'];
                $childSubCategory->image = $childSubCategoryData['image'];
                $childSubCategory->cat_id = $childSubCategoryData['cat_id'];
                $childSubCategory->date_added = $childSubCategoryData['date_added'];

                $childSubCategories[] = $childSubCategory;
            }

            return $childSubCategories;
        } catch (PDOException $e) {
            return "Error retrieving child subcategories: " . $e->getMessage();
        }
    }
}

class SubCategory extends dbConnection
{
    public $id;
    public $name;
    public $image;
    public $cat_id; // Category ID
    public $main_cat_id; // Main Category ID
    public $date_added;

    // Setter methods

    public function addSubCategoryToDB()
    {
        try {
            // Add subcategory information to the 'tabl_subcategories' table
            $sql = "INSERT INTO tabl_subcategories (name, image, cat_id, main_cat_id, date_added) 
                    VALUES (:name, :image, :cat_id, :main_cat_id, NOW())";
            $params = [':name' => $this->name, ':image' => $this->image, ':cat_id' => $this->cat_id, ':main_cat_id' => $this->main_cat_id];

            $this->dbQuery($sql, $params);

            $this->id = $this->dbInsertId(); // Get the ID of the newly inserted subcategory

            // Set date_added property
            $this->date_added = date("Y-m-d H:i:s");

            return $this->id; // Return the ID of the newly inserted subcategory
        } catch (PDOException $e) {
            return "Error adding subcategory: " . $e->getMessage();
        }
    }

    public function editSubCategoryInDB()
    {
        try {
            // Edit subcategory information in the 'tabl_subcategories' table
            $sql = "UPDATE tabl_subcategories 
                    SET name = :name, image = :image, cat_id = :cat_id, main_cat_id = :main_cat_id 
                    WHERE id = :id";
            $params = [
                ':id' => $this->id,
                ':name' => $this->name,
                ':image' => $this->image,
                ':cat_id' => $this->cat_id,
                ':main_cat_id' => $this->main_cat_id
            ];

            $this->dbQuery($sql, $params);

            return true; // Successfully edited the subcategory
        } catch (PDOException $e) {
            return "Error editing subcategory: " . $e->getMessage();
        }
    }

    public function deleteSubCategoryFromDB()
    {
        try {
            // Delete subcategory information from the 'tabl_subcategories' table
            $sql = "DELETE FROM tabl_subcategories WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true; // Successfully deleted the subcategory
        } catch (PDOException $e) {
            return "Error deleting subcategory: " . $e->getMessage();
        }
    }

    public function getAllSubCategoriesFromDB()
    {
        try {
            // Retrieve all subcategories from the 'tabl_subcategories' table
            $sql = "SELECT * FROM tabl_subcategories";
            $result = $this->dbQuery($sql);

            $subCategories = [];

            while ($subCategoryData = $this->dbFetchAssoc($result)) {
                $subCategory = new SubCategory();
                $subCategory->id = $subCategoryData['id'];
                $subCategory->name = $subCategoryData['name'];
                $subCategory->image = $subCategoryData['image'];
                $subCategory->cat_id = $subCategoryData['cat_id'];
                $subCategory->main_cat_id = $subCategoryData['main_cat_id'];
                $subCategory->date_added = $subCategoryData['date_added'];

                $subCategories[] = $subCategory;
            }

            return $subCategories;
        } catch (PDOException $e) {
            return "Error retrieving subcategories: " . $e->getMessage();
        }
    }

    public function getSubCategoryByIdFromDB($subCategoryId)
    {
        try {
            // Retrieve subcategory information from the 'tabl_subcategories' table by ID
            $sql = "SELECT * FROM tabl_subcategories WHERE id = :id";
            $params = [':id' => $subCategoryId];

            $result = $this->dbQuery($sql, $params);

            if ($result !== false) {
                $subCategoryData = $this->dbFetchAssoc($result);

                if ($subCategoryData) {
                    // Set subcategory data to class properties
                    $this->id = $subCategoryData['id'];
                    $this->name = $subCategoryData['name'];
                    $this->image = $subCategoryData['image'];
                    $this->cat_id = $subCategoryData['cat_id'];
                    $this->main_cat_id = $subCategoryData['main_cat_id'];
                    $this->date_added = $subCategoryData['date_added'];

                    return true; // Successfully retrieved subcategory data
                } else {
                    return "Subcategory not found";
                }
            } else {
                return "Error retrieving subcategory data";
            }
        } catch (PDOException $e) {
            return "Error retrieving subcategory data: " . $e->getMessage();
        }
    }
}












// // Assuming you have already included and instantiated the necessary classes
// $mainCategoryObj = new MainCategory();
// $categoryObj = new Category();
// $subCategoryObj = new SubCategory();

// // Example: Add a Main Category
// $mainCategoryObj->name = "Furniture";
// $mainCategoryObj->image = "furniture.jpg";
// $mainCategoryId = $mainCategoryObj->addMainCategoryToDB();
// echo "Main Category added with ID: $mainCategoryId<br>";

// // Example: Get Main Category by ID
// $mainCategoryObj->getMainCategoryByIdFromDB($mainCategoryId);
// echo "Main Category Name: " . $mainCategoryObj->name . "<br>";

// // Example: Edit Main Category
// $mainCategoryObj->name = "Furniture & Decor";
// $mainCategoryObj->image = "furniture_decor.jpg";
// $mainCategoryObj->editMainCategoryInDB();
// echo "Main Category edited successfully<br>";

// // Example: Get Child Categories of Main Category
// $childCategories = $mainCategoryObj->getChildCategoriesOfMainCatFromDB($mainCategoryId);
// foreach ($childCategories as $childCategory) {
//     echo "Child Category Name: " . $childCategory->name . "<br>";
// }

// // Example: Add a Category
// $categoryObj->name = "Living Room Furniture";
// $categoryObj->image = "living_room_furniture.jpg";
// $categoryObj->main_cat_id = $mainCategoryId;
// $categoryId = $categoryObj->addCategoryToDB();
// echo "Category added with ID: $categoryId<br>";

// // Example: Get Category by ID
// $categoryObj->getCategoryByIdFromDB($categoryId);
// echo "Category Name: " . $categoryObj->name . "<br>";

// // Example: Edit Category
// $categoryObj->name = "Living & Lounge Furniture";
// $categoryObj->image = "living_lounge_furniture.jpg";
// $categoryObj->editCategoryInDB();
// echo "Category edited successfully<br>";

// // Example: Get Child Subcategories of Category
// $childSubCategories = $categoryObj->getChildSubCategoriesOfCatFromDB($categoryId);
// foreach ($childSubCategories as $childSubCategory) {
//     echo "Child Subcategory Name: " . $childSubCategory->name . "<br>";
// }

// // Example: Add a Subcategory
// $subCategoryObj->name = "Sofas";
// $subCategoryObj->image = "sofas.jpg";
// $subCategoryObj->cat_id = $categoryId;
// $subCategoryId = $subCategoryObj->addSubCategoryToDB();
// echo "Subcategory added with ID: $subCategoryId<br>";

// // Example: Get Subcategory by ID
// $subCategoryObj->getSubCategoryByIdFromDB($subCategoryId);
// echo "Subcategory Name: " . $subCategoryObj->name . "<br>";
