<?php

class Coupon extends dbConnection
{
    public $id;
    public $code;
    public $discount;
    public $expiry_date;
    public $type; // Added type property
    public $date_added;

    public function addCouponToDB()
    {
        try {
            $sql = "INSERT INTO tabl_coupons (code, discount, expiry_date, type, date_added) 
                    VALUES (:code, :discount, :expiry_date, :type, NOW())";
            $params = [
                ':code' => $this->code,
                ':discount' => $this->discount,
                ':expiry_date' => $this->expiry_date,
                ':type' => $this->type
            ];

            $this->dbQuery($sql, $params);
            $this->id = $this->dbInsertId();

            return $this->id;
        } catch (PDOException $e) {
            return "Error adding coupon: " . $e->getMessage();
        }
    }

    public function editCouponInDB()
    {
        try {
            $sql = "UPDATE tabl_coupons SET code = :code, discount = :discount, expiry_date = :expiry_date, type = :type 
                    WHERE id = :id";
            $params = [
                ':id' => $this->id,
                ':code' => $this->code,
                ':discount' => $this->discount,
                ':expiry_date' => $this->expiry_date,
                ':type' => $this->type
            ];

            $this->dbQuery($sql, $params);

            return true;
        } catch (PDOException $e) {
            return "Error editing coupon: " . $e->getMessage();
        }
    }

    public function removeCouponFromDB()
    {
        try {
            $sql = "DELETE FROM tabl_coupons WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true;
        } catch (PDOException $e) {
            return "Error removing coupon: " . $e->getMessage();
        }
    }

    public function getCouponByIdFromDB($id)
    {
        try {
            $sql = "SELECT * FROM tabl_coupons WHERE id = :id";
            $params = [':id' => $id];

            $result = $this->dbQuery($sql, $params);

            return $this->dbFetchAssoc($result);
        } catch (PDOException $e) {
            return "Error retrieving coupon: " . $e->getMessage();
        }
    }

    public function getAllCouponsFromDB()
    {
        try {
            $sql = "SELECT * FROM tabl_coupons ORDER BY date_added DESC";
            $result = $this->dbQuery($sql);

            $coupons = [];

            while ($couponData = $this->dbFetchAssoc($result)) {
                $coupon = new Coupon();
                $coupon->id = $couponData['id'];
                $coupon->code = $couponData['code'];
                $coupon->discount = $couponData['discount'];
                $coupon->expiry_date = $couponData['expiry_date'];
                $coupon->type = $couponData['type'];
                $coupon->date_added = $couponData['date_added'];

                $coupons[] = $coupon;
            }

            return $coupons;
        } catch (PDOException $e) {
            return "Error retrieving coupons: " . $e->getMessage();
        }
    }
}
?>