<?php
class Coupon extends dbConnection
{
    public $id;
    public $title;
    public $code;
    public $discount;
    public $expiry_date;
    public $type;
    public $image;
    public $date_added;

    // Constructor and other functions go here...

    public function addCouponToDB()
    {
        try {
            $sql = "INSERT INTO tabl_coupons (title, code, discount, expiry_date, type, image, date_added) 
                    VALUES (:title, :code, :discount, :expiry_date, :type, :image, NOW())";

            $params = [
                ':title' => $this->title,
                ':code' => $this->code,
                ':discount' => $this->discount,
                ':expiry_date' => $this->expiry_date,
                ':type' => $this->type,
                ':image' => $this->image,
            ];

            $this->dbQuery($sql, $params);
            $this->id = $this->dbInsertId();
            $this->date_added = date("Y-m-d H:i:s");

            return $this->id;
        } catch (PDOException $e) {
            return "Error adding coupon: " . $e->getMessage();
        }
    }

    public function editCouponInDB()
    {
        try {
            $sql = "UPDATE tabl_coupons 
                    SET title = :title, code = :code, discount = :discount, expiry_date = :expiry_date, 
                        type = :type, image = :image, date_added = NOW()
                    WHERE id = :id";

            $params = [
                ':id' => $this->id,
                ':title' => $this->title,
                ':code' => $this->code,
                ':discount' => $this->discount,
                ':expiry_date' => $this->expiry_date,
                ':type' => $this->type,
                ':image' => $this->image,
            ];

            $this->dbQuery($sql, $params);
            $this->date_added = date("Y-m-d H:i:s");

            return true;
        } catch (PDOException $e) {
            return "Error updating coupon: " . $e->getMessage();
        }
    }

    public function deleteCouponFromDB()
    {
        try {
            $sql = "DELETE FROM tabl_coupons WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            // Remove the image file from the folder
            $this->removeCouponImage();

            return true;
        } catch (PDOException $e) {
            return "Error deleting coupon: " . $e->getMessage();
        }
    }

    public function getCouponByIdFromDB($id)
    {
        try {
            $sql = "SELECT * FROM tabl_coupons WHERE id = :id";
            $params = [':id' => $id];

            $result = $this->dbQuery($sql, $params);
            $couponData = $this->dbFetchAssoc($result);

            if ($couponData) {
                // Set class variables with retrieved data
                $this->id = $couponData['id'];
                $this->title = $couponData['title'];
                $this->code = $couponData['code'];
                $this->discount = $couponData['discount'];
                $this->expiry_date = $couponData['expiry_date'];
                $this->type = $couponData['type'];
                $this->image = $couponData['image'];

            }

            return true;

        } catch (PDOException $e) {
            return "Error retrieving coupon: " . $e->getMessage();
        }
    }

    public function getCouponsByTypeFromDB($type)
    {
        try {
            $sql = "SELECT * FROM tabl_coupons WHERE type = :type ORDER BY `date_added` DESC";
            $params = [':type' => $type];

            $result = $this->dbQuery($sql, $params);

            $coupons = [];

            while ($couponData = $this->dbFetchAssoc($result)) {
                $coupon = new Coupon();
                $coupon->id = $couponData['id'];
                $coupon->title = $couponData['title'];
                $coupon->code = $couponData['code'];
                $coupon->discount = $couponData['discount'];
                $coupon->expiry_date = $couponData['expiry_date'];
                $coupon->type = $couponData['type'];
                $coupon->image = $couponData['image'];
                $coupon->date_added = $couponData['date_added'];

                $coupons[] = $coupon;
            }

            return $coupons;
        } catch (PDOException $e) {
            return "Error retrieving coupons: " . $e->getMessage();
        }
    }

    public function getAllCouponsFromDB()
    {
        try {
            $sql = "SELECT * FROM tabl_coupons ORDER BY `date_added` DESC";
            $result = $this->dbQuery($sql);

            $coupons = [];

            while ($couponData = $this->dbFetchAssoc($result)) {
                $coupon = new Coupon();
                $coupon->id = $couponData['id'];
                $coupon->title = $couponData['title'];
                $coupon->code = $couponData['code'];
                $coupon->discount = $couponData['discount'];
                $coupon->expiry_date = $couponData['expiry_date'];
                $coupon->type = $couponData['type'];
                $coupon->image = $couponData['image'];
                $coupon->date_added = $couponData['date_added'];

                $coupons[] = $coupon;
            }

            return $coupons;
        } catch (PDOException $e) {
            return "Error retrieving coupons: " . $e->getMessage();
        }
    }

    private function removeCouponImage()
    {
        // Specify the path to the coupon image folder
        $imageFolder = "../../assets/images/coupons/";

        // Get the image filename from the database
        $imageFilename = $this->image;

        // Remove the image file from the folder
        if (!empty($imageFilename) && file_exists($imageFolder . $imageFilename)) {
            unlink($imageFolder . $imageFilename);
            unlink($imageFolder . "thumb-50/" . $imageFilename);
            unlink($imageFolder . "thumb-100/" . $imageFilename);
            unlink($imageFolder . "thumb-300/" . $imageFilename);
        }
    }
}
