<?php

// include_once 'dbConnection.php';

class Product extends dbConnection
{
    public $id;
    public $name;
    public $shortDescription;
    public $description;
    public $mrp;
    public $price;
    public $height;
    public $length;
    public $breadth;
    public $weight;
    public $mainCategory;
    public $category;
    public $subCategory;
    public $shippingPrice;
    public $image;
    public $gtsRate;       // New attribute: GTS rate
    public $hsnCode;       // New attribute: HSN code
    public $productCost;   // New attribute: Product cost
    public $shippingCost;  // New attribute: Shipping cost
    public $status;
    public $isTrending;
    public $isSpecial;
    public $isFeatured;
    public $date_added;     // New attribute: Date added
    public $date_updated;   // New attribute: Date updated

    // Setter methods (similar to previous version)

    // Add product to the database
    public function addProductToDB()
    {
        try {
            // $now = date("Y-m-d H:i:s");
            // $this->date_added = $now;
            // $this->date_updated = $now;

            $sql = "INSERT INTO tabl_products (name, short_description, description, mrp, price, height, length, breadth, weight, main_cat, cat, sub_cat, shipping_price, image, gts_rate, hsn_code, product_cost, shipping_cost, status, is_trending, is_special, is_featured, date_added, date_updated)
                    VALUES (:name, :shortDescription, :description, :mrp, :price, :height, :length, :breadth, :weight, :mainCategory, :category, :subCategory, :shippingPrice, :image, :gtsRate, :hsnCode, :productCost, :shippingCost, :status, :isTrending, :isSpecial, :isFeatured, :date_added, :date_updated)";
            $params = [
                // Existing attributes
                ':name' => $this->name,
                ':shortDescription' => $this->shortDescription,
                ':description' => $this->description,
                ':mrp' => $this->mrp,
                ':price' => $this->price,
                ':height' => $this->height,
                ':length' => $this->length,
                ':breadth' => $this->breadth,
                ':weight' => $this->weight,
                ':mainCategory' => $this->mainCategory,
                ':category' => $this->category,
                ':subCategory' => $this->subCategory,
                ':shippingPrice' => $this->shippingPrice,
                ':image' => $this->image,
                // New attributes
                ':gtsRate' => $this->gtsRate,
                ':hsnCode' => $this->hsnCode,
                ':productCost' => $this->productCost,
                ':shippingCost' => $this->shippingCost,
                ':status' => $this->status,
                ':isTrending' => $this->isTrending,
                ':isSpecial' => $this->isSpecial,
                ':isFeatured' => $this->isFeatured,
                ':date_added' => $this->date_added,
                ':date_updated' => $this->date_updated,
            ];

            // $statement = $this->dbQuery($sql, $params);
            // $statement->debugDumpParams();


            $this->dbQuery($sql, $params);

            return $this->dbInsertId(); // Return the ID of the newly inserted product
        } catch (PDOException $e) {
            return "Error adding product: " . $e->getMessage();
        }
    }

    // Edit product in the database
    public function editProductInDB()
    {
        try {
            $now = date("Y-m-d H:i:s");
            $sql = "UPDATE tabl_products SET
                    name = :name,
                    short_description = :shortDescription,
                    description = :description,
                    mrp = :mrp,
                    price = :price,
                    height = :height,
                    length = :length,
                    breadth = :breadth,
                    weight = :weight,
                    main_cat = :mainCategory,
                    cat = :category,
                    sub_cat = :subCategory,
                    shipping_price = :shippingPrice,
                    image = :image,
                    gts_rate = :gtsRate,
                    hsn_code = :hsnCode,
                    product_cost = :productCost,
                    shipping_cost = :shippingCost,
                    status = :status,
                    is_trending = :isTrending,
                    is_special = :isSpecial,
                    is_featured = :isFeatured,
                    date_added = :date_added,
                    date_updated = :date_updated
                    WHERE id = :id";
            $params = [
                // Existing attributes
                ':id' => $this->id,
                ':name' => $this->name,
                ':shortDescription' => $this->shortDescription,
                ':description' => $this->description,
                ':mrp' => $this->mrp,
                ':price' => $this->price,
                ':height' => $this->height,
                ':length' => $this->length,
                ':breadth' => $this->breadth,
                ':weight' => $this->weight,
                ':mainCategory' => $this->mainCategory,
                ':category' => $this->category,
                ':subCategory' => $this->subCategory,
                ':shippingPrice' => $this->shippingPrice,
                ':image' => $this->image,
                // New attributes
                ':gtsRate' => $this->gtsRate,
                ':hsnCode' => $this->hsnCode,
                ':productCost' => $this->productCost,
                ':shippingCost' => $this->shippingCost,
                ':status' => $this->status,
                ':isTrending' => $this->isTrending,
                ':isSpecial' => $this->isSpecial,
                ':isFeatured' => $this->isFeatured,
                ':date_added' => $this->date_added,
                ':date_updated' => $now,
            ];

            $this->dbQuery($sql, $params);

            return true; // Successfully edited the product
        } catch (PDOException $e) {
            return "Error editing product: " . $e->getMessage();
        }
    }

    // Retrieve product information from the database by ID
    public function getProductByIdFromDB($productId)
    {
        try {
            $sql = "SELECT * FROM tabl_products WHERE id = :id";
            $params = [':id' => $productId];

            $result = $this->dbQuery($sql, $params);

            $productData = $this->dbFetchAssoc($result);

            if ($productData) {
                // Set class variables with retrieved data
                $this->id = $productData['id'];
                $this->name = $productData['name'];
                $this->shortDescription = $productData['short_description'];
                $this->description = $productData['description'];
                $this->mrp = $productData['mrp'];
                $this->price = $productData['price'];
                $this->height = $productData['height'];
                $this->length = $productData['length'];
                $this->breadth = $productData['breadth'];
                $this->weight = $productData['weight'];
                $this->mainCategory = $productData['main_cat'];
                $this->category = $productData['cat'];
                $this->subCategory = $productData['sub_cat'];
                $this->shippingPrice = $productData['shipping_price'];
                $this->image = $productData['image'];
                // New attributes
                $this->gtsRate = $productData['gts_rate'];
                $this->hsnCode = $productData['hsn_code'];
                $this->productCost = $productData['product_cost'];
                $this->shippingCost = $productData['shipping_cost'];
                $this->status = $productData['status'];
                $this->isTrending = $productData['is_trending'];
                $this->isSpecial = $productData['is_special'];
                $this->isFeatured = $productData['is_featured'];
                $this->date_added = $productData['date_added'];
                $this->date_updated = $productData['date_updated'];

                return true; // Successfully retrieved product data
            } else {
                return "Product not found with ID: $productId";
            }
        } catch (PDOException $e) {
            return "Error retrieving product: " . $e->getMessage();
        }
    }

    // Get all products from the database
    public function getAllProducts()
    {
        try {
            $sql = "SELECT * FROM tabl_products"; // Update table name if needed
            $result = $this->dbQuery($sql);

            $products = [];

            while ($productData = $result->fetch(PDO::FETCH_ASSOC)) {
                $product = new Products();
                $product->id = $productData['id'];
                $product->name = $productData['name'];
                $product->shortDescription = $productData['short_description'];
                $product->description = $productData['description'];
                $product->mrp = $productData['mrp'];
                $product->price = $productData['price'];
                $product->height = $productData['height'];
                $product->length = $productData['length'];
                $product->breadth = $productData['breadth'];
                $product->weight = $productData['weight'];
                $product->mainCategory = $productData['main_cat'];
                $product->category = $productData['cat'];
                $product->subCategory = $productData['sub_cat'];
                $product->shippingPrice = $productData['shipping_price'];
                $product->image = $productData['image'];
                $product->gtsRate = $productData['gts_rate'];
                $product->hsnCode = $productData['hsn_code'];
                $product->productCost = $productData['product_cost'];
                $product->shippingCost = $productData['shipping_cost'];
                $product->status = $productData['status'];
                $product->isTrending = $productData['is_trending'];
                $product->isSpecial = $productData['is_special'];
                $product->isFeatured = $productData['is_featured'];
                $product->date_added = $productData['date_added'];
                $product->date_updated = $productData['date_updated'];

                $products[] = $product;
            }

            return $products;
        } catch (PDOException $e) {
            return "Error getting all products: " . $e->getMessage();
        }
    }

    public function deleteProductFromDB()
    {
        try {
            $sql = "DELETE FROM tabl_products WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true; // Successfully deleted the product
        } catch (PDOException $e) {
            return "Error deleting product: " . $e->getMessage();
        }
    }

    // Other getter and utility methods...
}




class ProductImages extends dbConnection
{
    public $id;
    public $product_id;
    public $image_path;
    public $date_added;

    public function addProductImageToDB()
    {
        try {
            $sql = "INSERT INTO tabl_product_images (product_id, image_path, date_added) VALUES (:product_id, :image_path, NOW())";
            $params = [':product_id' => $this->product_id, ':image_path' => $this->image_path];

            $this->dbQuery($sql, $params);

            $this->id = $this->dbInsertId();

            $this->date_added = date("Y-m-d H:i:s");

            return $this->id;
        } catch (PDOException $e) {
            return "Error adding product image: " . $e->getMessage();
        }
    }

    public function removeProductImageFromDB()
    {
        try {
            $sql = "DELETE FROM tabl_product_images WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true;
        } catch (PDOException $e) {
            return "Error removing product image: " . $e->getMessage();
        }
    }
}




class ProductFeatures extends dbConnection
{
    public $id;
    public $product_id;
    public $attribute;
    public $value;

    public function addProductFeatureToDB()
    {
        try {
            $sql = "INSERT INTO tabl_product_features (product_id, attribute, value) VALUES (:product_id, :attribute, :value)";
            $params = [':product_id' => $this->product_id, ':attribute' => $this->attribute, ':value' => $this->value];

            $this->dbQuery($sql, $params);

            $this->id = $this->dbInsertId();

            return $this->id;
        } catch (PDOException $e) {
            return "Error adding product feature: " . $e->getMessage();
        }
    }

    public function removeProductFeatureFromDB()
    {
        try {
            $sql = "DELETE FROM tabl_product_features WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true;
        } catch (PDOException $e) {
            return "Error removing product feature: " . $e->getMessage();
        }
    }
}





// Example usage:
// $dbConnection = new dbConnection();
// $product = new Products();

// // Set product data
// $product->name = "Sample Product";
// $product->price = 99.99;

// echo $product->price;

// Add product to the database
// $newProductId = $product->addProductToDB();

// Update product in the database
// $product->id = $newProductId; // Set the product ID to the one you want to update
// $product->name = "Updated Product Name";
// $product->editProductInDB();

// Delete product from the database
// $product->id = $newProductId; // Set the product ID to the one you want to delete
// $product->deleteProductFromDB();



// $productImage = new ProductImages();
// $productImage->product_id = 1;
// $productImage->image_path = 'hi';
// $productImage->addProductImageToDB();
