<?php
class Product extends dbConnection
{
    public function addProduct($name, $description, $price, $quantity)
    {
        try {
            $sql = "INSERT INTO products (name, description, price, quantity) VALUES (:name, :description, :price, :quantity)";
            $params = [
                ':name' => $name,
                ':description' => $description,
                ':price' => $price,
                ':quantity' => $quantity
            ];

            $this->dbQuery($sql, $params);

            return $this->dbInsertId(); // Return the ID of the newly inserted product
        } catch (PDOException $e) {
            return "Error adding product: " . $e->getMessage();
        }
    }

    public function editProduct($productId, $name, $description, $price, $quantity)
    {
        try {
            $sql = "UPDATE products SET name = :name, description = :description, price = :price, quantity = :quantity WHERE id = :id";
            $params = [
                ':id' => $productId,
                ':name' => $name,
                ':description' => $description,
                ':price' => $price,
                ':quantity' => $quantity
            ];

            $this->dbQuery($sql, $params);

            return true; // Successfully edited the product
        } catch (PDOException $e) {
            return "Error editing product: " . $e->getMessage();
        }
    }

    public function deleteProduct($productId)
    {
        try {
            $sql = "DELETE FROM products WHERE id = :id";
            $params = [':id' => $productId];

            $this->dbQuery($sql, $params);

            return true; // Successfully deleted the product
        } catch (PDOException $e) {
            return "Error deleting product: " . $e->getMessage();
        }
    }

    public function getProductById($productId)
    {
        try {
            $sql = "SELECT * FROM products WHERE id = :id";
            $params = [':id' => $productId];

            $result = $this->dbQuery($sql, $params);

            return $this->dbFetchAssoc($result);
        } catch (PDOException $e) {
            return "Error retrieving product: " . $e->getMessage();
        }
    }

    public function getAllProducts()
    {
        try {
            $sql = "SELECT * FROM products";

            $result = $this->dbQuery($sql);

            return $this->dbFetchAssoc($result);
        } catch (PDOException $e) {
            return "Error retrieving products: " . $e->getMessage();
        }
    }
}
