<?php

class User extends dbConnection
{
    public $id;
    public $username;
    public $email;
    public $phone;
    public $password;
    public $first_name;
    public $last_name;
    public $date_of_birth;
    public $status;
    public $date_added; // Property to store the date_added
    public $date_updated; // Property to store the date_updated
    public $addresses = []; // Array to store user addresses

    // Setter methods (similar to previous version)

    public function addUserToDB()
    {
        try {
            // Add user information to the 'tabl_users' table
            $sql = "INSERT INTO tabl_users (username, email, phone, password, first_name, last_name, date_of_birth, status, date_added, date_updated)
                    VALUES (:username, :email, :phone, :password, :first_name, :last_name, :date_of_birth, :status, NOW(), NOW())";
            $params = [
                ':username' => $this->username,
                ':email' => $this->email,
                ':phone' => $this->phone,
                ':password' => $this->password,
                ':first_name' => $this->first_name,
                ':last_name' => $this->last_name,
                ':date_of_birth' => $this->date_of_birth,
                ':status' => $this->status,
            ];

            $this->dbQuery($sql, $params);

            $this->id = $this->dbInsertId(); // Get the ID of the newly inserted user

            // Set date_added and date_updated properties
            $this->date_added = date("Y-m-d H:i:s");
            $this->date_updated = date("Y-m-d H:i:s");

            // Add user addresses to the 'tabl_addresses' table
            foreach ($this->addresses as $address) {
                $address->user_id = $this->id; // Set the user_id for each address
                $address->addAddressToDB();
            }

            return $this->id; // Return the ID of the newly inserted user
        } catch (PDOException $e) {
            return "Error adding user: " . $e->getMessage();
        }
    }

    public function editUserInDB()
    {
        try {
            // Edit user information in the 'tabl_users' table
            $sql = "UPDATE tabl_users SET username = :username, email = :email, phone = :phone, password = :password, first_name = :first_name, last_name = :last_name, date_of_birth = :date_of_birth, status = :status, date_updated = NOW() WHERE id = :id";
            $params = [
                ':id' => $this->id,
                ':username' => $this->username,
                ':email' => $this->email,
                ':phone' => $this->phone,
                ':password' => $this->password,
                ':first_name' => $this->first_name,
                ':last_name' => $this->last_name,
                ':date_of_birth' => $this->date_of_birth,
                ':status' => $this->status,
            ];

            $this->dbQuery($sql, $params);

            // Set date_updated property
            $this->date_updated = date("Y-m-d H:i:s");

            // Edit user addresses in the 'tabl_addresses' table
            foreach ($this->addresses as $address) {
                $address->editAddressInDB();
            }

            return true; // Successfully edited the user
        } catch (PDOException $e) {
            return "Error editing user: " . $e->getMessage();
        }
    }

    public function deleteUserFromDB()
    {
        try {
            // Delete user addresses in the 'tabl_addresses' table
            foreach ($this->addresses as $address) {
                $address->deleteAddressFromDB();
            }

            // Delete user information from the 'tabl_users' table
            $sql = "DELETE FROM tabl_users WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true; // Successfully deleted the user
        } catch (PDOException $e) {
            return "Error deleting user: " . $e->getMessage();
        }
    }

    public function getUserByIdFromDB($userId)
    {
        try {
            // Retrieve user information from the 'tabl_users' table
            $sql = "SELECT * FROM tabl_users WHERE id = :id";
            $params = [':id' => $userId];

            $result = $this->dbQuery($sql, $params);

            if ($result !== false) {
                $userData = $this->dbFetchAssoc($result);

                if ($userData) {
                    // Set user data to class properties
                    $this->id = $userData['id'];
                    $this->username = $userData['username'];
                    $this->email = $userData['email'];
                    $this->phone = $userData['phone'];
                    $this->password = $userData['password'];
                    $this->first_name = $userData['first_name'];
                    $this->last_name = $userData['last_name'];
                    $this->date_of_birth = $userData['date_of_birth'];
                    $this->status = $userData['status'];
                    $this->date_added = $userData['date_added'];
                    $this->date_updated = $userData['date_updated'];

                    // Retrieve user addresses from the 'tabl_addresses' table
                    $this->addresses = $this->getAddressesByUserIdFromDB($this->id);

                    return true; // Successfully retrieved user data
                } else {
                    return "User not found";
                }
            } else {
                return "Error retrieving user data";
            }
        } catch (PDOException $e) {
            return "Error retrieving user data: " . $e->getMessage();
        }
    }

    private function getAddressesByUserIdFromDB($userId)
    {
        try {
            // Retrieve user addresses from the 'tabl_addresses' table
            $sql = "SELECT * FROM tabl_addresses WHERE user_id = :user_id";
            $params = [':user_id' => $userId];

            $result = $this->dbQuery($sql, $params);

            $addresses = [];

            while ($addressData = $this->dbFetchAssoc($result)) {
                $address = new Address();
                $address->id = $addressData['id'];
                $address->user_id = $addressData['user_id'];
                $address->street = $addressData['street'];
                $address->city = $addressData['city'];
                $address->state = $addressData['state'];
                $address->zip_code = $addressData['zip_code'];
                $address->country = $addressData['country'];
                $address->date_added = $addressData['date_added'];
                $address->date_updated = $addressData['date_updated'];

                $addresses[] = $address;
            }

            return $addresses;
        } catch (PDOException $e) {
            return "Error retrieving addresses: " . $e->getMessage();
        }
    }


    public function getAllUsersFromDB()
    {
        try {
            // Retrieve all users from the 'tabl_users' table
            $sql = "SELECT * FROM tabl_users";
            $result = $this->dbQuery($sql);

            $users = [];

            while ($userData = $this->dbFetchAssoc($result)) {
                $user = new User();
                $user->id = $userData['id'];
                $user->username = $userData['username'];
                $user->email = $userData['email'];
                $user->password = $userData['password'];
                $user->first_name = $userData['first_name'];
                $user->last_name = $userData['last_name'];
                $user->date_of_birth = $userData['date_of_birth'];
                $user->date_added = $userData['date_added'];
                $user->date_updated = $userData['date_updated'];

                // Retrieve user addresses from the 'tabl_addresses' table
                $user->addresses = $this->getAddressesByUserIdFromDB($user->id);

                $users[] = $user;
            }

            return $users;
        } catch (PDOException $e) {
            return "Error retrieving users: " . $e->getMessage();
        }
    }

}

class Address extends dbConnection
{
    public $id;
    public $user_id;
    public $street;
    public $city;
    public $state;
    public $zip_code;
    public $country;
    public $date_added; // Property to store the date_added
    public $date_updated; // Property to store the date_updated

    // Setter methods (similar to previous version)

    public function addAddressToDB()
    {
        try {
            $sql = "INSERT INTO tabl_addresses (user_id, street, city, state, zip_code, country, date_added, date_updated)
                    VALUES (:user_id, :street, :city, :state, :zip_code, :country, NOW(), NOW())";
            $params = [
                ':user_id' => $this->user_id,
                ':street' => $this->street,
                ':city' => $this->city,
                ':state' => $this->state,
                ':zip_code' => $this->zip_code,
                ':country' => $this->country,
            ];

            $this->dbQuery($sql, $params);

            // Set date_added and date_updated properties
            $this->date_added = date("Y-m-d H:i:s");
            $this->date_updated = date("Y-m-d H:i:s");

            return $this->dbInsertId(); // Return the ID of the newly inserted address
        } catch (PDOException $e) {
            return "Error adding address: " . $e->getMessage();
        }
    }

    public function editAddressInDB()
    {
        try {
            $sql = "UPDATE tabl_addresses SET user_id = :user_id, street = :street, city = :city, state = :state, zip_code = :zip_code, country = :country, date_updated = NOW() WHERE id = :id";
            $params = [
                ':id' => $this->id,
                ':user_id' => $this->user_id,
                ':street' => $this->street,
                ':city' => $this->city,
                ':state' => $this->state,
                ':zip_code' => $this->zip_code,
                ':country' => $this->country,
            ];

            $this->dbQuery($sql, $params);

            // Set date_updated property
            $this->date_updated = date("Y-m-d H:i:s");

            return true; // Successfully edited the address
        } catch (PDOException $e) {
            return "Error editing address: " . $e->getMessage();
        }
    }

    public function deleteAddressFromDB()
    {
        try {
            // Delete user address from the 'tabl_addresses' table
            $sql = "DELETE FROM tabl_addresses WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true; // Successfully deleted the address
        } catch (PDOException $e) {
            return "Error deleting address: " . $e->getMessage();
        }
    }

    // ... (remaining methods remain unchanged)
}


// Example usage:
// $dbConnection = new dbConnection();
$user = new User();

// Set user data
// $user->username = "john_doe";
// $user->email = "john.doe@example.com";
// $user->password = password_hash("secure_password", PASSWORD_DEFAULT);
// $user->first_name = "John";
// $user->last_name = "Doe";
// $user->date_of_birth = "1990-01-01";

// Set user addresses
// $address1 = new Address();
// $address1->street = "123 Main Street";
// $address1->city = "Cityville";
// $address1->state = "State";
// $address1->zip_code = "12345";
// $address1->country = "Country";

// $address2 = new Address();
// $address2->street = "456 Oak Avenue";
// $address2->city = "Townsville";
// $address2->state = "State";
// $address2->zip_code = "67890";
// $address2->country = "Country";

// $user->addresses = [$address1, $address2];

// Add user to the database
// $newUserId = $user->addUserToDB();

// Update user in the database
// $user->id = $newUserId; // Set the user ID to the one you want to update
// $user->username = "updated_john_doe";
// $user->editUserInDB();

// Delete user from the database
// $user->id = $newUserId; // Set the user ID to the one you want to delete
// $user->deleteUserFromDB();



// Get user data by ID
// $userId = 1; // Set the ID of the user you want to retrieve
// $result = $user->getUserByIdFromDB($userId);

// if ($result === true) {
//     // User data is now available in the class variables
//     echo "User ID: " . $user->id . PHP_EOL;
//     echo "User Name: " . $user->username . PHP_EOL;
//     echo "User Email: " . $user->email . PHP_EOL;
//     echo "User Addresses:" . PHP_EOL;

//     foreach ($user->addresses as $address) {
//         echo " - " . $address->street . ", " . $address->city . ", " . $address->state . " " . $address->zip_code . ", " . $address->country . PHP_EOL;
//     }
// } else {
//     echo $result; // Display error message if user retrieval fails
// }
