<?php

include_once 'dbConnection.php';

class User extends dbConnection
{
    public $id;
    public $username;
    public $email;
    public $password;
    public $first_name;
    public $last_name;
    public $date_of_birth;
    public $addresses = []; // Array to store user addresses

    // Setter methods (similar to previous version)

    public function addUserToDB()
    {
        try {
            // Add user information to the 'users' table
            $sql = "INSERT INTO users (username, email, password, first_name, last_name, date_of_birth)
                    VALUES (:username, :email, :password, :first_name, :last_name, :date_of_birth)";
            $params = [
                ':username' => $this->username,
                ':email' => $this->email,
                ':password' => $this->password,
                ':first_name' => $this->first_name,
                ':last_name' => $this->last_name,
                ':date_of_birth' => $this->date_of_birth,
            ];

            $this->dbQuery($sql, $params);

            $this->id = $this->dbInsertId(); // Get the ID of the newly inserted user

            // Add user addresses to the 'addresses' table
            foreach ($this->addresses as $address) {
                $address->user_id = $this->id; // Set the user_id for each address
                $address->addAddressToDB();
            }

            return $this->id; // Return the ID of the newly inserted user
        } catch (PDOException $e) {
            return "Error adding user: " . $e->getMessage();
        }
    }

    public function editUserInDB()
    {
        try {
            // Edit user information in the 'users' table
            $sql = "UPDATE users SET username = :username, email = :email, password = :password, first_name = :first_name, last_name = :last_name, date_of_birth = :date_of_birth WHERE id = :id";
            $params = [
                ':id' => $this->id,
                ':username' => $this->username,
                ':email' => $this->email,
                ':password' => $this->password,
                ':first_name' => $this->first_name,
                ':last_name' => $this->last_name,
                ':date_of_birth' => $this->date_of_birth,
            ];

            $this->dbQuery($sql, $params);

            // Edit user addresses in the 'addresses' table
            foreach ($this->addresses as $address) {
                $address->editAddressInDB();
            }

            return true; // Successfully edited the user
        } catch (PDOException $e) {
            return "Error editing user: " . $e->getMessage();
        }
    }

    public function deleteUserFromDB()
    {
        try {
            // Delete user addresses from the 'addresses' table
            foreach ($this->addresses as $address) {
                $address->deleteAddressFromDB();
            }

            // Delete user information from the 'users' table
            $sql = "DELETE FROM users WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true; // Successfully deleted the user
        } catch (PDOException $e) {
            return "Error deleting user: " . $e->getMessage();
        }
    }

    public function getUserByIdFromDB($userId)
    {
        try {
            // Retrieve user information from the 'users' table
            $sql = "SELECT * FROM users WHERE id = :id";
            $params = [':id' => $userId];

            $result = $this->dbQuery($sql, $params);

            $userData = $this->dbFetchAssoc($result);

            if ($userData) {
                // Set class variables with retrieved user data
                $this->id = $userData['id'];
                $this->username = $userData['username'];
                $this->email = $userData['email'];
                $this->password = $userData['password'];
                $this->first_name = $userData['first_name'];
                $this->last_name = $userData['last_name'];
                $this->date_of_birth = $userData['date_of_birth'];

                // Retrieve user addresses from the 'addresses' table
                $this->addresses = $this->getAddressesByUserId($userId);

                return true; // Successfully retrieved user data
            } else {
                return "User not found with ID: $userId";
            }
        } catch (PDOException $e) {
            return "Error retrieving user: " . $e->getMessage();
        }
    }

    // Retrieve user addresses by user ID
    private function getAddressesByUserId($userId)
    {
        $addresses = [];
        $sql = "SELECT * FROM addresses WHERE user_id = :user_id";
        $params = [':user_id' => $userId];

        $result = $this->dbQuery($sql, $params);

        while ($addressData = $this->dbFetchAssoc($result)) {
            $address = new Address();
            $address->id = $addressData['id'];
            $address->user_id = $addressData['user_id'];
            $address->street = $addressData['street'];
            $address->city = $addressData['city'];
            $address->state = $addressData['state'];
            $address->zip_code = $addressData['zip_code'];
            $address->country = $addressData['country'];

            $addresses[] = $address;
        }

        return $addresses;
    }
}

class Address extends dbConnection
{
    public $id;
    public $user_id;
    public $street;
    public $city;
    public $state;
    public $zip_code;
    public $country;

    // Setter methods (similar to previous version)

    public function addAddressToDB()
    {
        try {
            $sql = "INSERT INTO addresses (user_id, street, city, state, zip_code, country)
                    VALUES (:user_id, :street, :city, :state, :zip_code, :country)";
            $params = [
                ':user_id' => $this->user_id,
                ':street' => $this->street,
                ':city' => $this->city,
                ':state' => $this->state,
                ':zip_code' => $this->zip_code,
                ':country' => $this->country,
            ];

            $this->dbQuery($sql, $params);

            return $this->dbInsertId(); // Return the ID of the newly inserted address
        } catch (PDOException $e) {
            return "Error adding address: " . $e->getMessage();
        }
    }

    public function editAddressInDB()
    {
        try {
            $sql = "UPDATE addresses SET user_id = :user_id, street = :street, city = :city, state = :state, zip_code = :zip_code, country = :country WHERE id = :id";
            $params = [
                ':id' => $this->id,
                ':user_id' => $this->user_id,
                ':street' => $this->street,
                ':city' => $this->city,
                ':state' => $this->state,
                ':zip_code' => $this->zip_code,
                ':country' => $this->country,
            ];

            $this->dbQuery($sql, $params);

            return true; // Successfully edited the address
        } catch (PDOException $e) {
            return "Error editing address: " . $e->getMessage();
        }
    }

    public function deleteAddressFromDB()
    {
        try {
            $sql = "DELETE FROM addresses WHERE id = :id";
            $params = [':id' => $this->id];

            $this->dbQuery($sql, $params);

            return true; // Successfully deleted the address
        } catch (PDOException $e) {
            return "Error deleting address: " . $e->getMessage();
        }
    }
}

// Example usage:
$dbConnection = new dbConnection();
$user = new User();

// Set user data
$user->username = "john_doe";
$user->email = "john.doe@example.com";
$user->password = password_hash("secure_password", PASSWORD_DEFAULT);
$user->first_name = "John";
$user->last_name = "Doe";
$user->date_of_birth = "1990-01-01";

// Set user addresses
$address1 = new Address();
$address1->street = "123 Main Street";
$address1->city = "Cityville";
$address1->state = "State";
$address1->zip_code = "12345";
$address1->country = "Country";

$address2 = new Address();
$address2->street = "456 Oak Avenue";
$address2->city = "Townsville";
$address2->state = "State";
$address2->zip_code = "67890";
$address2->country = "Country";

$user->addresses = [$address1, $address2];

// Add user to the database
$newUserId = $user->addUserToDB();

// Update user in the database
$user->id = $newUserId; // Set the user ID to the one you want to update
$user->username = "updated_john_doe";
$user->editUserInDB();

// Delete user from the database
$user->id = $newUserId; // Set the user ID to the one you want to delete
$user->deleteUserFromDB();

// Get user data by ID
$userId = 1; // Set the ID of the user you want to retrieve
$result = $user->getUserByIdFromDB($userId);

if ($result === true) {
    // User data is now available in the class variables
    echo "User ID: " . $user->id . PHP_EOL;
    echo "User Name: " . $user->username . PHP_EOL;
    echo "User Email: " . $user->email . PHP_EOL;
    echo "User Addresses:" . PHP_EOL;

    foreach ($user->addresses as $address) {
        echo " - " . $address->street . ", " . $address->city . ", " . $address->state . " " . $address->zip_code . ", " . $address->country . PHP_EOL;
    }
} else {
    echo $result; // Display error message if user retrieval fails
}
