<?php
define('HOSTNAME', 'localhost');
define('USERNAME', 'root');
define('PASSWORD', '');
define('DATABASE_NAME', 'kri_biter');

try {
	// Create a PDO connection
	$pdo = new PDO("mysql:host=" . HOSTNAME . ";dbname=" . DATABASE_NAME, USERNAME, PASSWORD);

	// Set PDO to throw exceptions for errors
	$pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
	echo "Failed to connect to MySQL: " . $e->getMessage();
}

function dbQuery($sql, $params = [])
{
	global $pdo;

	try {
		// Prepare the SQL statement
		$stmt = $pdo->prepare($sql);

		// Bind parameters if provided
		foreach ($params as $paramName => $paramValue) {
			// echo '<br>' . $paramName;
			// echo '<br>' . $paramValue;
			// $stmt->bindParam($paramName, $paramValue);
			$stmt->bindValue($paramName, $paramValue);
		}

		// echo $stmt->queryString;
		// $boundSql = substituteParams($stmt->queryString, $params);
		// echo $boundSql;

		// Execute the query
		$stmt->execute();

		// Return the result set
		return $stmt;
	} catch (PDOException $e) {
		// Handle any exceptions that occur during the process
		// You might want to log the error or return an error message
		return "Error: " . $e->getMessage();
	}
}


function substituteParams($sql, $params)
{
	foreach ($params as $paramName => $paramValue) {
		// Replace placeholders with actual parameter values
		$sql = str_replace($paramName, $paramValue, $sql);
	}
	return $sql;
}



// function dbAffectedRows()
// {
// 	global $pdo;
// 	return $pdo->rowCount();
// }

function dbFetchAssoc($result)
{
	return $result->fetch(PDO::FETCH_ASSOC);
}

function dbFetchRow($result)
{
	return $result->fetch(PDO::FETCH_NUM);
}

function dbNumRows($result)
{
	if (is_string($result)) {
		return 0;
	}

	return $result->rowCount();
}

function dbInsertId()
{
	global $pdo;
	return $pdo->lastInsertId();
}


function update_wallet($admin_id, $amount, $type)
{
	try {
		$amount_new = ($type == 'credit') ? check_wallet($admin_id) + $amount : check_wallet($admin_id) - $amount;

		// Use dbQuery for the update
		dbQuery("UPDATE tabl_wallet SET amount = :amount_new WHERE user_id = :user_id", [
			':amount_new' => $amount_new,
			':user_id' => $admin_id
		]);
	} catch (PDOException $e) {
		return "Error: " . $e->getMessage();
	}
}

function check_wallet($admin_id)
{
	try {
		// Use dbQuery for the select
		$result = dbQuery("SELECT amount FROM tabl_wallet WHERE user_id = :user_id", [':user_id' => $admin_id]);

		// Fetch the result as an associative array
		$res = dbFetchAssoc($result);

		return $res['amount'];
	} catch (PDOException $e) {
		return "Error: " . $e->getMessage();
	}
}

function update_wallet_balance($admin_id, $amount)
{
	try {
		// Use dbQuery for the update
		dbQuery("UPDATE tabl_wallet SET amount = :amount_new WHERE user_id = :user_id", [
			':amount_new' => $amount,
			':user_id' => $admin_id
		]);
	} catch (PDOException $e) {
		return "Error: " . $e->getMessage();
	}
}


function removeImageFile($imagePath)
{
	// echo $imagePath;
	// Ensure the image path is valid and not empty
	if (!empty($imagePath) && file_exists($imagePath)) {
		// Remove the image file from the folder
		unlink($imagePath);
	}
}




$setting = dbQuery("SELECT * FROM tabl_setting WHERE id=1");
$res_setting = dbFetchAssoc($setting);

define("SITE", $res_setting['site_name']);
define("SITE_LOGO", $res_setting['site_logo']);
define("EMAIL", $res_setting['site_email']);
define("PHONE", $res_setting['site_phone']);
define("ADDRESS", $res_setting['site_address']);



// Example usage
// $admin_id = 1;
// $credit_amount = 50;
// update_wallet($admin_id, $credit_amount, 'credit');

// Example usage
// $admin_id = 1;
// $new_balance = check_wallet($admin_id);
// echo "New Wallet Balance: $new_balance";


// // Example usage
// $tableName = "your_table";
// $insertData = [
// 	'column1' => 'example_value1',
// 	'column2' => 'example_value2',
// 	'column3' => 'example_value3',
// ];

// $result = dbQuery($tableName, $insertData);
// echo $result;




function uploadImage($file, $target_dir)
{
	require_once('resize-class.php');

	$originalFileName = $file["name"];
	$extension = pathinfo($originalFileName, PATHINFO_EXTENSION);
	$new_name = date("Ymd") . "_" . rand(100000, 999999) . "." . $extension;
	$uploadPath = $target_dir . $new_name;

	if (move_uploaded_file($file["tmp_name"], $uploadPath)) {

		$resizeObj = new resize($uploadPath);

		 // Thumbnail (50x50)
		 $resizeObj->resizeImage(50, 50, 'exact');
		 $resizeObj->saveImage($target_dir . "thumb-50/" . $new_name, 100);
 
		 // Thumbnail (100x100)
		 $resizeObj->resizeImage(100, 100, 'exact');
		 $resizeObj->saveImage($target_dir . "thumb-100/" . $new_name, 100);
 
		 // Thumbnail (300x300)
		 $resizeObj->resizeImage(300, 300, 'exact');
		 $resizeObj->saveImage($target_dir . "thumb-300/" . $new_name, 100);
 
		 // Thumbnail (500x500, cropped)
		 $resizeObj->resizeImage(579, 581, 'crop');
		 $resizeObj->saveImage($target_dir . "thumb-500/" . $new_name, 100);
		return $new_name;
	}

	return '';
}





class dbConnection
{
	private $pdo;

	public function __construct()
	{

		try {
			// Create a PDO connection
			$this->pdo = new PDO("mysql:host=" . HOSTNAME . ";dbname=" . DATABASE_NAME, USERNAME, PASSWORD);

			// Set PDO to throw exceptions for errors
			$this->pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		} catch (PDOException $e) {
			echo "Failed to connect to MySQL: " . $e->getMessage();
			exit(); // Terminate the script if the connection fails
		}
	}

	public function connection($sql){
		$stmt = $this->pdo = new PDO("mysql:host=" . HOSTNAME . ";dbname=" . DATABASE_NAME, USERNAME, PASSWORD);

		return $this->pdo->prepare($sql);
	}
	public function dbQuery($sql, $params = [])
	{
		try {
			// Prepare the SQL statement
			$stmt = $this->pdo->prepare($sql);

			// Bind parameters if provided
			foreach ($params as $paramName => $paramValue) {
				$stmt->bindValue($paramName, $paramValue);
			}


			// echo $stmt->queryString;
			// $boundSql = substituteParams($stmt->queryString, $params);
			// echo $boundSql;


			// Execute the query
			$stmt->execute();

			// Return the result set
			return $stmt;
		} catch (PDOException $e) {
			// Handle any exceptions that occur during the process
			// You might want to log the error or return an error message
			return "Error: " . $e->getMessage();
		}
	}

	public function dbFetchAssoc($result)
	{
		return $result->fetch(PDO::FETCH_ASSOC);
	}
	public function dbFetchObject($result)
	{
		return $result->fetch(PDO::FETCH_OBJ);
	}


	public function dbFetchAllAssoc($result)
	{
		return $result->fetchAll(PDO::FETCH_ASSOC);
	}


	public function dbInsertId()
	{
		return $this->pdo->lastInsertId();
	}
}



// include("./Product.php");

// require_once 'User.php';
// require_once 'Categories.php';
// require_once 'Products.php';
// require_once 'Banner.php';


// Example usage:
// $dbConnection = new dbConnection();
// $products = new Products();

// Now you can use $products to perform CRUD operations on products.
